@@ -1,11 +1,14 @@
 ## -*- truncate-lines: t; -*-
-## Copyright (C) 2018  Enrico Schumann
+## Copyright (C) 2018-20  Enrico Schumann
 
 streaks <- function(x, ...)
     UseMethod("streaks")
 
-streaks.default <- function(x, up =  0.2, down = -0.2,
-                            initial.state = NA, ...) {
+streaks.default <- function(x,
+                            up =  0.2, down = -up,
+                            initial.state = NA,
+                            y = NULL,
+                            ...) {
 
     start <- 1
     end <- NA
@@ -13,34 +16,34 @@ streaks.default <- function(x, up =  0.2, down = -0.2,
     results <- data.frame(start = numeric(0),
                           end = numeric(0),
                           state = character(0))
-
+    if (is.null(y)) y <- rep.int(1, length(x))
     if (is.na(state)) {
-        hi <- x[1]
-        lo <- x[1]
+        hi <- x[1]/y[1]
+        lo <- x[1]/y[1]
         hi.t <- 1
         lo.t <- 1
     } else if (state == "up") {
-        hi <- x[1]
+        hi <- x[1]/y[1]
         lo <- NA
         hi.t <- 1
         lo.t <- NA
     } else if (state == "down") {
         hi <- NA
-        lo <- x[1]
+        lo <- x[1]/y[1]
         hi.t <- NA
         lo.t <- 1
     }
     for (t in 2:length(x)) {
-        dx <- x[t] - x[t - 1]
-        x.i <- x[t]
+        dx <- x[t]/x[t - 1] / (y[t]/y[t - 1])
+        xy.i <- x[t]/y[t]
         if (is.na(state)) {
-            if (dx >= 0) {
+            if (dx >= 1) {
 
-                if (x.i > hi) {
-                    hi <- x.i
+                if (xy.i > hi) {
+                    hi <- xy.i
                     hi.t <- t
                 }
-                if (x.i/lo - 1 >= up) {
+                if ( (x[t]/x[lo.t]) / (y[t]/y[lo.t]) - 1 >= up) {
                     state <- "up"
                     if (lo.t == 1) {
                         lo <- NA
@@ -55,13 +58,13 @@ streaks.default <- function(x, up =  0.2, down = -0.2,
                     }
                 }
 
-            } else if (dx < 0) {
+            } else if (dx < 1) {
 
-                if (x.i < lo) {
-                    lo <- x.i
+                if (xy.i < lo) {
+                    lo <- xy.i
                     lo.t <- t
                 }
-                if (x.i/hi - 1 <= down) {
+                if ( (x[t]/x[hi.t]) / (y[t]/y[hi.t]) - 1 <= down) {
                     state <- "down"
                     if (hi.t == 1) {
                         hi <- NA
@@ -72,17 +75,19 @@ streaks.default <- function(x, up =  0.2, down = -0.2,
                                          data.frame(start = 1,
                                                     end = hi.t,
                                                     state = NA))
-                        start <- hi.t }}
+                        start <- hi.t
+                    }
+                }
             }
 
         } else if (state == "up") {
 
-            if (dx >= 0) {
-                if (x.i > hi) {
-                    hi <- x.i
+            if (dx >= 1) {
+                if (xy.i > hi) {
+                    hi <- xy.i
                     hi.t <- t
                 }
-            } else if (dx < 0 && x.i/hi -1 < down) {
+            } else if (dx < 1 && (x[t]/x[hi.t]) / (y[t]/y[hi.t]) - 1 < down) {
                 results <- rbind(results,
                                  data.frame(start = start,
                                             end = hi.t,
@@ -90,19 +95,19 @@ streaks.default <- function(x, up =  0.2, down = -0.2,
                 state <- "down"
                 start <- hi.t
                 lo.t <- t
-                lo <- x.i
+                lo <- xy.i
                 hi.t <- NA
                 hi <- NA
             }
 
         } else if (state == "down") {
 
-            if (dx <= 0) {
-                if (x.i < lo) {
-                    lo <- x.i
+            if (dx <= 1) {
+                if (xy.i < lo) {
+                    lo <- xy.i
                     lo.t <- t
                 }
-            } else if (dx > 0 && x.i/lo -1 > up) {
+            } else if (dx > 1 && (x[t]/x[lo.t]) / (y[t]/y[lo.t]) - 1 > up) {
                 results <- rbind(results,
                                  data.frame(start = start,
                                             end = lo.t,
@@ -112,7 +117,7 @@ streaks.default <- function(x, up =  0.2, down = -0.2,
                 lo.t <- NA
                 lo <- NA
                 hi.t <- t
-                hi <- x.i
+                hi <- xy.i
             }
 
         }
@@ -121,20 +126,22 @@ streaks.default <- function(x, up =  0.2, down = -0.2,
                      data.frame(start = start,
                                 end = length(x),
                                 state = state))
-    results[["return"]] <- x[results$end]/x[results$start] - 1
+    results[["return"]] <- x[results$end]/x[results$start] /
+                          (y[results$end]/y[results$start]) - 1
     results
 }
 
 streaks.zoo <- function(x,
                         up   =  0.2,
-                        down = -0.2,
-                        initial.state = NA, ...) {
+                        down = -up,
+                        initial.state = NA,
+                        y = NULL, ...) {
     t <- index(x)
-    x <- coredata(x)
-    ans <- streaks.default(x,
+    ans <- streaks.default(x = coredata(x),
                            up   = up,
                            down = down,
-                           initial.state = initial.state, ...)
+                           initial.state = initial.state,
+                           y = coredata(y), ...)
     ans$start <- t[ans$start]
     ans$end   <- t[ans$end]
     ans
@@ -142,10 +149,11 @@ streaks.zoo <- function(x,
 
 streaks.NAVseries <- function(x,
                               up   =  0.2,
-                              down = -0.2,
-                              initial.state = NA, ...) {
+                              down = -up,
+                              initial.state = NA,
+                              bm = NULL, ...) {
     xx <- as.zoo(x)
     streaks.zoo(xx,
                 up   = up, down = down,
-                initial.state = initial.state, ...)
+                initial.state = initial.state, y = bm, ...)
 }
